!     Beispielprogramm zur VDI Richtlinie 3783 Blatt 16
!     
!     Autor: Dr. Thomas Flassak, Ingenieurbro Lohmeyer GmbH & Co. KG
!
!     Aufgabe: Filtert im Randbereich 2dx-Wellen heraus.
!
!     Methode: Mehrfaches Anwenden von 
!                Neuer Hhenwert = 50% Alter Hhenwert + 25% Alter Nachbar-Hhenwert1 + 25% Alter Nachbar-Hhenwert2
!              An Nord- bzw. Sdrndern: Nachbar-Hhenwert1 = Hhenwert am Ort (j,i-1) und Nachbar-Hhenwert2 = Hhenwert am Ort (j,i+1)  
!              Am West- bzw. Ostrndern: Nachbar-Hhenwert1 = Hhenwert am Ort (j-1,i) und Nachbar-Hhenwert2 = Hhenwert am Ort (j+1,i)  
!
!     - Gewichtungsfaktor nimmt mit der Entfernung vom Rand linear von 1 auf 0 ab. 
!     - Es wird nur senkrecht zum Rand geglttet
!
!     - Die hier vorliegende Beispielrealisierung liest ein METRAS-PC-Topfile ein und schreibt die gefilterten Daten im METRAS-PC-Format wieder aus.
!
!     - Speziell fr METRAS-PC gilt, dass die jeweils 3 uersten Hhenwerte senkrecht zum Rand den gleichen Hhenwert haben. 
!
!     Last revision: 18.06.2013 by TF

      character*256 input,output,chdummy,Zeile
      character*36  Zeile36
      
      real,allocatable :: H(:,:), Hmod(:,:)
      real,allocatable :: YDX(:,:),YDY(:,:)
      real,allocatable :: yzz(:)      
      
      logical istda
      
      open(50,file='Filter2dxnurRand.log')      
      
! Lesen der Inputparameter aus Datei 'Filter2dxnurRand.inp': 
!    - METRAS-PC-Top-Inputdatei (Variable input)
!    - Ausgabedatei mit gefilterten Hhen (im METRAS-PC-Format, Variable output) 
!    - Anzahl der randparallelen Randreihen auf die der Filter angewendet wird (Variable iAnzahlKragenreihen) 
!    - Anzahl der Glttungsdurchlufe (Variable iAnzahlGlaettungsdurchlaeufe)  

!===== Fehlerbehandlung ======      
      inquire(FILE = 'Filter2dxnurRand.inp',EXIST=istda)
      if (.not.istda) then
        write(50,*) 'FEHLER: Datei Filter2dxnurRand.inp nicht da.'
        stop
      end if
      
      open(10,file='Filter2dxnurRand.inp')
      read(10,'(A256)',END=997,ERR=998) input
      read(10,'(A256)',END=997,ERR=998) output      
      read(10,*       ,END=997,ERR=999) iAnzahlKragenreihen ! wird in VDI 3783 Blatt mit der Variable N bezeichnet.
      read(10,*       ,END=997,ERR=999) iAnzahlGlaettungsdurchlaeufe      
    
      write(50,*) ' METRAS-PC Input-Top-Datei             : ', input(1:len_trim(input))      
      write(50,*) ' Geglttete METRAS-PC Output-Top-Datei : ', output(1:len_trim(output))            
      write(50,*) ' Anzahl Kragenreihen                   : ', iAnzahlKragenreihen                  
      write(50,*) ' Anzahl der Glttungsdurchlufe        : ', iAnzahlGlaettungsdurchlaeufe
      
      xAnzahlKragenreihen=REAL(iAnzahlKragenreihen)
      
!===== Fehlerbehandlung ======      
      inquire(FILE = input,EXIST=istda)
      if (.not.istda) then
        write(50,*) 'FEHLER: Datei ',input(1:len_trim(input)),' existiert nicht'
        stop
      end if
      
      if (input.eq.output) then
        write(50,*) 'FEHLER: input = output unzulssig'
        stop
      end if
      
      if (iAnzahlGlaettungsdurchlaeufe.lt.10) then
        write(50,*) 'FEHLER: Anzahl der Glaettungsdurchlaeufe (=Variable iAnzahlGlaettungsdurchlaeufe) muss grer gleich 10 sein'
        stop  
      end if
      if (iAnzahlKragenreihen.lt.10) then
        write(50,*) 'FEHLER: Anzahl randparalleler Reihen auf die Filter angewandt wird (=Variable iAnzahlKragenreihen) muss grer gleich 10 sein'
        stop  
      end if

!===== METRAS input top file einlesen ========      
      open(10,file=input)
      read(10,*) chdummy,Nz,chdummy,Ny,chdummy,Nx
      
      if ((iAnzahlKragenreihen.ge.Nx/2) .or. (iAnzahlKragenreihen.ge.Ny/2))  then
        write(50,*) 'FEHLER: Kragenbereich (=Variable iAnzahlKragenreihen) zu gro gewhlt (grer als Nx/2 oder Ny/2)'
        stop  
      end if
      
      write(50,*) ' Nx,Ny,Nz aus METRAS-Top-Datei: ',Nx,Ny,Nz

!==============================================      
      ! In METRAS-PC luft z.B. der Index in y-Richtung von 0 bis NY+1. Die Reihen j=0 und 1 sind Randreihen mit gleicher Gelndehhe wie j=2. 
      ! Gefiltert wird fr j = 2,iAnzahlKragenreihen+1. In der VDI-Richtlinie 3783 Blatt 16 luft der Filter jedoch von j=1,iAnzahlKragenreihen. 
      ! Deswegen wird um Konsistenz mit der Formel in VDI Richtlinie 3783 Blatt 16 herzustellen NX bzw. NY um 2 verringert 
      ! und die entsprechenden Felder nicht von j=0,NY+1 sondern von j=-1,NY+2 eingelesen.
      ! Fr den Index i gilt analoges.
      NX=NX-2
      NY=NY-2
!==============================================      
      
      allocate (H  (-1:Nx+2,-1:Ny+2),Hmod(-1:Nx+2,-1:Ny+2))   
      allocate (YDX(-1:Nx+2,-1:Ny+2),YDY (-1:Nx+2,-1:Ny+2))      
      allocate (yzz(-1:NZ+1))      
      
      read(10,*)
      read(10,*) chdummy, drehwinkel
      read(10,*)
      read(10,*) chdummy,(yzz(k),k=-1,nz+1)
      read(10,'(2(7x,f9.0))') yxmin, yymin
      read(10,*)
      
      do j = -1,Ny+2
        do i = -1,Nx+2
          read(10,*) II,IJ,YDX(i,j),YDY(i,j), H(i,j)
        end do
      end do  
      close(10)
      write(50,*) ' METRAS-PC input top file erfolgreich gelesen'
      
      write(50,*) ' Drehwinkel  : ', drehwinkel
      write(50,*) ' yxmin, yymin: ', yxmin, yymin
      
! ========================================================================================      
! Einlesen abgeschlossen. 
! Anfang Filteralgorithmus nach 3783 Blatt 16

    do it = 1,iAnzahlGlaettungsdurchlaeufe

      Hmod=H
      
      !write(50,'(1000F8.2)') (Hmod(1,j),j=-1,Ny+2)
      
! === x-Richtung am Sdrand ==========================================  
 
      do j = 1,iAnzahlKragenreihen
      
        alpha=(xAnzahlKragenreihen+1.-j)/xAnzahlKragenreihen
        !write(50,*) 'j,fak: ',j,alpha


        do i = 1,Nx
          Hmod(i,j)=(1.-alpha)*H(i,j) + alpha*(0.25*H(i-1,j)+0.5*H(i,j)+0.25*H(i+1,j))
        end do
        
        
        !Am Westrand horizontal ergnzen
        Hmod(0   ,j)=Hmod(1 ,j)
        Hmod(-1  ,j)=Hmod(1 ,j) 
        !Am Ostrand horizontal ergnzen        
        Hmod(Nx+1,j)=Hmod(Nx,j)
        Hmod(Nx+2,j)=Hmod(Nx,j)        
      end do
      
      !Randreihen am Sdrand horizontal ergnzen
      Hmod(:,-1)=Hmod(:,1)
      Hmod(:, 0)=Hmod(:,1)
      
      !write(50,'(1000F8.2)') (Hmod(1,j),j=-1,Ny+2)
      
! === x-Richtung am Nordrand ==========================================        
 
      do j = Ny-iAnzahlKragenreihen+1,Ny
       
        alpha=(j-(Ny-xAnzahlKragenreihen))/xAnzahlKragenreihen
        
        do i = 1,Nx
          Hmod(i,j)=(1.-alpha)*H(i,j) + alpha*(0.25*H(i-1,j)+0.5*H(i,j)+0.25*H(i+1,j))
        end do
        
        !Am Westrand horizontal ergnzen        
        Hmod(0   ,j)=Hmod(1 ,j)
        Hmod(-1  ,j)=Hmod(1 ,j)        
        !Am Ostrand horizontal ergnzen                
        Hmod(Nx+1,j)=Hmod(Nx,j)
        Hmod(Nx+2,j)=Hmod(Nx,j)        
      end do  
      
      !Randreihen am Nordrand horizontal ergnzen
      Hmod(:,Ny+2)=Hmod(:,Ny)
      Hmod(:,Ny+1)=Hmod(:,Ny)
      
      H=Hmod
      
! === y-Richtung am Westrand ==========================================   
 
      !write(50,*) '=== y-Richtung ===='
      do i = 1,iAnzahlKragenreihen      
       
        alpha=(xAnzahlKragenreihen+1.-i)/xAnzahlKragenreihen

        do j = 1,Ny
          Hmod(i,j)=(1.-alpha)*H(i,j) + alpha*(0.25*H(i,j-1)+0.5*H(i,j)+0.25*H(i,j+1))
        end do
        
        !Am Sdrand horizontal ergnzen 
        Hmod(i,0   )=Hmod(i,1 )
        Hmod(i,-1  )=Hmod(i,1 )        
        !Am Nordrand horizontal ergnzen
        Hmod(i,Ny+1)=Hmod(i,Ny)
        Hmod(i,Ny+2)=Hmod(i,Ny)                
      end do  
      
      !Randreihen am Westrand horizontal ergnzen
      Hmod(-1,:)=Hmod(1,:)
      Hmod( 0,:)=Hmod(1,:)


! === y-Richtung am Ostrand ==========================================   

      do i = Nx-iAnzahlKragenreihen+1,Nx      
       
        alpha=(i-(Nx-xAnzahlKragenreihen))/xAnzahlKragenreihen

        do j = 1,Ny
          Hmod(i,j)=(1.-alpha)*H(i,j) + alpha*(0.25*H(i,j-1)+0.5*H(i,j)+0.25*H(i,j+1))
        end do

        !Am Sdrand horizontal ergnzen 
        Hmod(i,0   )=Hmod(i,1 )
        Hmod(i,-1  )=Hmod(i,1 )
        !Am Nordrand horizontal ergnzen        
        Hmod(i,Ny+1)=Hmod(i,Ny)
        Hmod(i,Ny+2)=Hmod(i,Ny)                
      end do  

      !Randreihen am Ostrand horizontal ergnzen  
      Hmod(Nx+2,:)=Hmod(Nx,:)
      Hmod(Nx+1,:)=Hmod(Nx,:)
      
      H=Hmod
      
    end do
      
! Ende Filteralgorithmus nach 3783 Blatt 16

!  Gefiltertes Ergebnis wieder im METRAS-PC-Format ausschreiben    
    
    open(11,file=output,ERR=996)
    open(10,file=input)
      
    iz= (nz+3) / 5
    if (mod(nz+3,5).gt.0) iz=iz+1
    iz=iz+6
      
    do i = 1, iz
      read(10,'(A256)',end=99999) Zeile
      write(11,'(A)') Zeile(1:len_trim(Zeile))
    end do  
      
    do j = -1,Ny+2
      do i = -1,Nx+2
        read(10,'(A256)',end=99999) Zeile        
        write(Zeile36,'(2I4,1X,2(F9.0,1X),F7.2)') i+1, j+1, YDX(i,j),YDY(i,j), Hmod(i,j)
        zeile=Zeile36//Zeile(37:256) 
        write(11,'(A)') Zeile(1:len_trim(Zeile))          
      end do
    end do  
      
99999 close(10)
    close(11) 
    write(50,*) ' Geglttete METRAS-PC-Top-Datei erfolgreich geschrieben!'
      
    stop 
    
! ===== Hier regulres Ende, im folgenden noch Meldungen bei Fehlern beim Lesen der Datei Filter2dxnurRand.inp.

996     write(50,*) 'FEHLER: Geglttete METRAS-PC-Top-Datei kann nicht geschrieben werden'
        stop
    
997     write(50,*) 'FEHLER in Datei Filter2dxnurRand.inp: Zu wenig Datenzeilen'
        stop
        
998     write(50,*) 'FEHLER in Datei Filter2dxnurRand.inp: Ungltiges Datenformat fr Dateinamen'
        stop

999     write(50,*) 'FEHLER in Datei Filter2dxnurRand.inp: Ungltiges Datenformat fr Integerzahlen'
        stop

    
    end